"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlaylistAudioStream = void 0;
var operators_1 = require("rxjs/operators");
var rxjs_audio_stream_1 = require("./rxjs-audio-stream");
var PlaylistAudioStream = /** @class */ (function (_super) {
    __extends(PlaylistAudioStream, _super);
    function PlaylistAudioStream(tracks, config) {
        var _this = _super.call(this, config) || this;
        _this.state = {
            playing: false,
            isFirstTrack: false,
            isLastTrack: false,
            trackInfo: {
                currentTrack: undefined,
                duration: undefined,
                currentTime: undefined
            }
        };
        _this.currentTrackIndex = 0;
        _this.tracks = [];
        _this.configureStream(config, tracks);
        return _this;
    }
    /**
     * Select the Next Audio Item in Playlist.
     * But it doens't play it automatically.
     * Run `play()` to play the stream
     */
    PlaylistAudioStream.prototype.next = function () {
        if (!this.isLastPlaying()) {
            this.loadTrackAtIndex(++this.currentTrackIndex);
        }
    };
    /**
     * Select the Previous Audio Item in Playlist.
     * But it doens't play it automatically.
     * Run `play()` to play the stream
     */
    PlaylistAudioStream.prototype.previous = function () {
        if (!this.isFirstPlaying()) {
            this.loadTrackAtIndex(--this.currentTrackIndex);
        }
    };
    /**
     * Method to Check if First Track is Playing or not
     * Alternatively you can `this.state()` Observable to get state.
     * state has `isFirstTrack:boolean` to check if First Track is playing or not
     */
    PlaylistAudioStream.prototype.isFirstPlaying = function () {
        if (this.currentTrackIndex === 0) {
            return true;
        }
        else {
            return false;
        }
    };
    /**
     * Method to Check if Last Track is Playing or not.
     * Alternatively you can `this.state()` Observable to get state.
     * state has `isLastTrack:boolean` to check if First Track is playing or not
     */
    PlaylistAudioStream.prototype.isLastPlaying = function () {
        if (this.currentTrackIndex === this.tracks.length - 1) {
            return true;
        }
        else {
            return false;
        }
    };
    /**
     * Method to play track at particular index
     */
    PlaylistAudioStream.prototype.playAt = function (index) {
        if (index < this.tracks.length) {
            this.loadTrackAtIndex(index);
            this.play();
        }
    };
    /**
     * Switch the currrent Track to particular index
     * It just switch the track, doesn't play the track.
     * To do switch and play, use `platAt()` method.
     */
    PlaylistAudioStream.prototype.switchTo = function (index) {
        if (index < this.tracks.length) {
            this.loadTrackAtIndex(index);
        }
    };
    /**
     * initialize the audio stream
     * internal implementation, don't use it.
     */
    PlaylistAudioStream.prototype.loadTrackAtIndex = function (index) {
        this.currentTrackIndex = index;
        this.loadTrack(this.tracks[index]);
        this.updateStateProps(this.state, 'isFirstTrack', this.isFirstPlaying());
        this.updateStateProps(this.state, 'isLastTrack', this.isLastPlaying());
    };
    /**
     * autoplay the next track if set in config
     * internal implementation, don't use it.
     */
    PlaylistAudioStream.prototype.autoPlayNext = function (config) {
        if (config && config.autoPlayNext) {
            this.next();
            this.play();
        }
    };
    /**
     * config the stream for initialization
     * internal implementation, don't use it.
     */
    PlaylistAudioStream.prototype.configureStream = function (config, tracks) {
        var _this = this;
        var initialTrack;
        /*  extract tracks from a list of track */
        this.tracks = tracks;
        /* set initial track based on user input */
        if (config && config.initialTrack) {
            initialTrack = config.initialTrack;
        }
        else {
            initialTrack = 0;
        }
        /** if there is no config, do the following  */
        if (!config) {
            initialTrack = 0;
            this.tracks = tracks;
        }
        /* initialize audio **/
        this.loadTrackAtIndex(initialTrack);
        /** if autoPlayNext is set to true */
        var canPlay$ = this.events().pipe(operators_1.filter(function (event) { return event.type === 'canplay'; }));
        var ended$ = this.events().pipe(operators_1.filter(function (event) { return event.type === 'ended'; }));
        canPlay$.pipe(operators_1.switchMap(function (event) { return ended$; })).subscribe(function (value) { return _this.autoPlayNext(config); });
    };
    return PlaylistAudioStream;
}(rxjs_audio_stream_1.AudioStream));
exports.PlaylistAudioStream = PlaylistAudioStream;
//# sourceMappingURL=rxjs-playlist-audio-stream.js.map