import { Observable, Subject } from 'rxjs';
import { Stream, StreamConfig, StreamState } from './rxjs-audio.interfaces';
export declare class AudioStream implements Stream {
    protected state: StreamState;
    private audio;
    private audioEvents;
    private stateChange;
    private config;
    constructor(config?: StreamConfig);
    /**
     * Play the Audio Stream
     */
    play(): void;
    /**
     * Pause the Audio Stream
     */
    pause(): void;
    /**
     * Stop the Audio Stream
     */
    stop(): void;
    /**
     * Seek audio to particular time
     *
     * @param time
     */
    seekTo(time: number): void;
    /**
     * Set the audio stream volume
     *
     * @param volume
     */
    setVolume(volume: number): void;
    /**
     * Mute the audio stream volume
     *
     * @param mute
     */
    setMute(muted: boolean): void;
    /**
     * Set sinkId for this stream
     *
     * @param mute
     */
    setSinkId(deviceId: string): void;
    /**
     * Returns Observable which listens to audio events of audio stream
     */
    events(): Observable<any>;
    /**
     * Method to get stream of state updates
     */
    getState(): Subject<any>;
    /**
     * load a new track
     */
    loadTrack(src: string): void;
    /**
     * method to add events listener for audio stream
     * internal implementation, don't use it.
     */
    protected addEvents(obj: any, events: Array<string>, handler: any): void;
    /**
     * method to remove events listeners for audio stream
     * internal implementation, don't use it.
     */
    protected removeEvents(obj: any, events: Array<string>, handler: any): void;
    /**
     * method to update state from audio events
     * internal implementation, don't use it.
     */
    protected updateStateEvents(event: Event): void;
    /**
     * method to update state from properties
     * internal implementation, don't use it.
     */
    protected updateStateProps(obj: any, prop: string, value: any): void;
}
